@@ -59,6 +59,7 @@ BEGIN {
           get_deb_info
           get_dsc_info
           get_file_checksum
+          get_file_digest
           slurp_entire_file
           file_is_encoded_in_non_utf8
           is_string_utf8_encoded
@@ -97,6 +98,7 @@ use Digest::SHA;
 use Encode ();
 use File::Temp qw(tempfile);
 use FileHandle;
+use List::Util qw(shuffle);
 use Scalar::Util qw(openhandle);
 
 use Lintian::Command qw(spawn safe_qx);
@@ -361,10 +363,10 @@ An invalid or malformed PGP header appears.
 
 Two "BEGIN PGP MESSAGE" headers appears in the same message.
 
-=item End of file but expected a "END PGP SIGNATURE" header
+=item End of file but expected an "END PGP SIGNATURE" header
 
 The file ended after a "BEGIN PGP SIGNATURE" header without being
-followed by a "END PGP SIGNATURE".
+followed by an "END PGP SIGNATURE".
 
 =item PGP MESSAGE header must be first content if present
 
@@ -440,7 +442,7 @@ sub visit_dpkg_paragraph {
             # message format identical.
             die join(q{ },
                 "syntax error at line $.:",
-                qq{End of file but expected a "END PGP SIGNATURE" header\n})
+                qq{End of file but expected an "END PGP SIGNATURE" header\n})
               unless $saw_end;
         }
         # other pgp control?
@@ -709,7 +711,7 @@ L</parse_dpkg_control> do.  It can also emit:
         # control files.
         drain_pipe($opts->{pipe_out});
         close($opts->{pipe_out});
-        $opts->{harness}->finish();
+        $opts->{harness}->finish;
         return $data[0];
     }
 }
@@ -717,7 +719,7 @@ L</parse_dpkg_control> do.  It can also emit:
 =item get_dsc_control (DSCFILE)
 
 Convenience function for reading dsc files.  It will read the DSCFILE
-using L<read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
+using L</read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
 first paragraph.  If the file has no paragraphs, C<undef> is returned
 instead.
 
@@ -785,7 +787,34 @@ sub drain_pipe {
     return 1;
 }
 
-=item get_file_checksum (ALGO, FILE)
+=item get_file_digest(ALGO, FILE)
+
+Creates an ALGO digest object that is seeded with the contents of
+FILE.  If you just want the hex digest, please use
+L</get_file_checksum(ALGO, FILE)> instead.
+
+ALGO can be 'md5' or shaX, where X is any number supported by
+L<Digest::SHA> (e.g. 'sha256').
+
+This sub is a convenience wrapper around Digest::{MD5,SHA}.
+
+=cut
+
+sub get_file_digest {
+    my ($alg, $file) = @_;
+    open(my $fd, '<', $file);
+    my $digest;
+    if ($alg eq 'md5') {
+        $digest = Digest::MD5->new;
+    } elsif ($alg =~ /sha(\d+)/) {
+        $digest = Digest::SHA->new($1);
+    }
+    $digest->addfile($fd);
+    close($fd);
+    return $digest;
+}
+
+=item get_file_checksum(ALGO, FILE)
 
 Returns a hexadecimal string of the message digest checksum generated
 by the algorithm ALGO on FILE.
@@ -798,16 +827,7 @@ This sub is a convenience wrapper around Digest::{MD5,SHA}.
 =cut
 
 sub get_file_checksum {
-    my ($alg, $file) = @_;
-    open(my $fd, '<', $file);
-    my $digest;
-    if ($alg eq 'md5') {
-        $digest = Digest::MD5->new;
-    } elsif ($alg =~ /sha(\d+)/) {
-        $digest = Digest::SHA->new($1);
-    }
-    $digest->addfile($fd);
-    close($fd);
+    my $digest = get_file_digest(@_);
     return $digest->hexdigest;
 }
 
@@ -1614,11 +1634,15 @@ harness state, return a list of group ids that are part of the
 backlog.  The list is sorted based on what version of Lintian
 processed the package.
 
+Note the result is by design not deterministic to reduce the
+risk of all large packages being in the same run (e.g. like
+gcc-5 + gcc-5-cross + gcc-6 + gcc-6-cross).
+
 =cut
 
 sub find_backlog {
     my ($lintian_version, $state) = @_;
-    my (@list, @sorted);
+    my (@backlog, %by_version);
     for my $group_id (keys(%{$state->{'groups'}})) {
         my $last_version = '0';
         my $group_data = $state->{'groups'}{$group_id};
@@ -1631,12 +1655,12 @@ sub find_backlog {
         }
         $is_out_of_date = 1
           if not versions_equal($last_version, $lintian_version);
-        push(@list, [$group_id, $last_version]) if $is_out_of_date;
+        push(@{$by_version{$last_version}}, $group_id) if $is_out_of_date;
     }
-    @sorted = map { $_->[0] }
-      sort { versions_comparator($a->[1], $b->[1]) || $a->[0] cmp $b->[0] }
-      @list;
-    return @sorted;
+    for my $v (sort(versions_comparator keys(%by_version))) {
+        push(@backlog, shuffle(@{$by_version{$v}}));
+    }
+    return @backlog;
 }
 
 =item untaint(VALUE)
